%% Paper: VEXPA: Validated EXPonential Analysis through regular sub-sampling.pdf
%
% Matteo Briani, Annie Cuyt and Wen-shin Lee 
%
% VEXPA: Validated EXPonential Analysis through regular 
% sub-sampling
%
% High noise experiment 5.2.
%

%% Script environment
% This script does depend on the random number generator state.

clear  
close all

%% High noise experiment 5.2.
% For our second experiment we consider a signal $\phi(t)$ defined by
% the parameters $\beta_i,\gamma_i, \omega_i, \psi_i$, $i=1,\dots,12$ 
% listed in Table 2. We perturb the samples with white circular Gaussian 
% noise of increasing SNR. The total number of samples is again N = 300. 
% The perturbed signal is then analysed using ESPRIT on the one hand 
% and VEXPA (on top of ESPRIT) on the other. For the latter we
% choose $u = 7$ and $s = 6$. We pass the correct model order $n$
% only to ESPRIT. The new VEXPA will detect it automatically.
%
% Up to SNR = 20 dB both ESPRIT and VEXPA closely approach the theoretical
% CRLB, as can be seen from Figure 10. In Figure 11 we show the retrieved $\omega_i$-values
% for both ESPRIT (top) and VEXPA (bottom). For higher noise levels (smaller
% SNR) the stand-alone ESPRIT method returns unreliable results, while the VEXPA
% method implemented on top of ESPRIT detects that the signal is heavily perturbed
% as none of the computed results is validated in the cluster analysis. Therefore
% VEXPA, in its standard implementation, does not return $\lambda_i$ output.

params = params_from_table2;
signal = params.construct(300);
w = imag((log(params.b)*100))/(2*pi);

bsolver_esprit = BSolverEsprit('--nsamples',300,'--ncols',100,...
                               '--nrows', 201,'--nterms',12);

u = 7;
s = 6;
bsolver_vexpa = BSolverVexpa( '--bsolver',   BSolverEsprit( ...
                                         '--nterms',15) ...
                        , '--csolver',   CSolverVandermondeLS ...
                        , '--rate'   ,   u ...
                        , '--shift'  ,   s ...
                        , '--M'      ,   8 ...
                        , '--u-epsilon', 0.1 ...
                        , '--u-minpts' , 6 ...
                        , '--s-epsilon', 0.1 ...
                        , '--s-minpts' , 5 ...
                        , '--plot'   , false ...
                        , '--time'   , false ... 
                        );
                    
SNRvec = 0:50;
N = numel(SNRvec);
nexpvec = [repmat(300,1,20),repmat(100,1,31)];
exp_total = sum(nexpvec); 

X_esprit = [];
Y_esprit = [];
X_vexpa = [];
Y_vexpa = [];
sigma = zeros(1,N);
crlb_esprit = zeros(1,N);
crlb_vexpa = zeros(1,N);
timer = 0;
for j = 1:N
    SNR = SNRvec(j);
    fprintf('\nSNR = %d\n', SNR)
    nexp = nexpvec(j);
    sigma_tmp = zeros(1,nexp);
    omrt_esprit = zeros(12,nexp);
    omrt_vexpa = zeros(12,nexp);
    for k = 1:nexp
        tic
        flag = 50;
        if ~mod(k,flag)
            ETA = exp_total*(timer/flag); 
            fprintf('   > Experiment %d/%d   > ETA: %d sec\n',...
                    k,nexp,round(ETA))
            timer = 0;
        end
        signal.remove_noise;
        sigma_tmp(k) = signal.add_white_gaussian_noise(SNR,'db');
        
        b_esprit = bsolver_esprit.solve(signal);
        w_esprit = imag(log(b_esprit)*100)/(2*pi);
        [~,I] = min(abs(w(:)-w_esprit),[],2);
        omrt_esprit(:,k) = w_esprit(I);
        X_esprit = [X_esprit,repmat(SNR,1,numel(b_esprit))];
        Y_esprit = [Y_esprit,w_esprit];
        
        b_vexpa = bsolver_vexpa.solve(signal);
        w_vexpa = imag(log(b_vexpa)*100)/(2*pi);
        if isempty(w_vexpa)
            w_vexpa = 0;
        end
        [~,I] = min(abs(w(:)-w_vexpa),[],2);
        omrt_vexpa(:,k) = w_vexpa(I);
        X_vexpa = [X_vexpa,repmat(SNR,1,numel(w_vexpa))];
        Y_vexpa = [Y_vexpa,w_vexpa];
        
        exp_total = exp_total-1;
        timer = timer + toc;
    end
    
    sigma(j) = mean(sigma_tmp);
    
    crlb_esprit(j)=sqrt(median(var(2*pi/100*omrt_esprit,[],2).^2));
    crlb_vexpa(j)=sqrt(median(var(2*pi/100*omrt_vexpa,[],2).^2));
end

crlb_theory = zeros(1,N);
for k = 1:N
    [~,~,crlb_params,~] = params.crlb(sigma(k),300);
    crlb_theory(k) = sqrt(median(crlb_params.^2));
end
figure
semilogy(SNRvec,crlb_theory,'k-')
hold on
semilogy(SNRvec,crlb_esprit,'b-s')
semilogy(SNRvec,crlb_vexpa,'r-o')
title(['Figure 10. Variance of ESPRIT (blue) and VEXPA (red),'...
       ' compared to the Cramer-Rao lower bound.'])

figure;
plot(repmat(SNRvec(:),1,12),repmat(w,N,1),'k')
hold on
ylim([-50,50])
xlabel('SNR_{dB}')
ylabel('|\omega_i / 2\pi |')
title('Figure 11. (top, blue) Retrieved \omega_i by ESPRIT.')
scatter(X_esprit,Y_esprit,20,'filled','MarkerFaceAlpha',.1,...
        'MarkerFaceColor','b')
title('Figure 11. (top, blue) Retrieved \omega_i by ESPRIT.')


figure;
plot(repmat(SNRvec(:),1,12),repmat(w,N,1),'k')
hold on
ylim([-50,50])
xlabel('SNR_{dB}')
ylabel('|\omega_i / 2\pi |')
title('Figure 11. (top, red) Retrieved \omega_i by VEXPA.')
scatter(X_vexpa,Y_vexpa,15,'filled','MarkerFaceAlpha',.1,...
        'MarkerFaceColor','r')
title('Figure 11. (bottom, red) Retrieved \omega_i by VEXPA.')