%% Paper: VEXPA: Validated EXPonential Analysis through regular sub-sampling.pdf
%
% Matteo Briani, Annie Cuyt and Wen-shin Lee 
%
% VEXPA: Validated EXPonential Analysis through regular 
% sub-sampling
%
% Example Figure 3.
%

%% Script environment
% This script depends on the random number generator state.

clear  
close all

%% Example Figure 3.
% We use the following notation: $\phi(t) = \sum\limits_{j=1}^n \alpha_i
% \exp(\mu_i t)$, where $\alpha_i = \beta_i \exp(\mathrm{i}\gamma_i)$ and
% $\mu_i = \psi_i + \mathrm{i}\omega_i$.
%
% We consider the practical computation of the CRLB provided in [24] and illustrate
% the relationship between the CRLB and the disposedness $\rho_i$ of $\lambda_i, i = 1, \dots , n$.
% Take the same toy example and add white circular Gaussian noise of varying signal
% to noise ratio (SNR). In Figure 3 we graph the root mean square of the vector of
% CRLB's for the parameters $\omega_i, i = 1, \dots , 10$, and this for decreasing SNR in three
% different situations: $\Delta = 1/\Omega$ with $N=200$ samples $\phi_j$ (blue
% triangles), $\Delta = 10/\Omega$ with $N=200$ samples $\phi_j$ (green
% squares) and $\Delta = 10/\Omega$ with $N=20$ samples $\phi_j$ (red circles).
% Note that multiplying $\Delta$ by $u = 10$ while maintaining $N = 200$ implies that the
% signal is sampled over a larger time interval, while multiplying $\Delta$ by $u = 10$ and
% dividing $N$ by $u = 10$ does not enlarge the observation window.

b = exp(2*pi*1i*(0:9)/100);
bu = exp(2*pi*1i*(0:9)/10);
c = ones(size(b));

params1 = MultiExponentialParameters(100,{b,c},'normalized');
params2 = MultiExponentialParameters(10,{bu,c},'normalized');
params3 = MultiExponentialParameters(10,{bu,c},'normalized');

nsamples = [200,200,20];
signal1 = params1.construct(nsamples(1));
signal2 = params2.construct(nsamples(2));
signal3 = params3.construct(nsamples(3));

SNR = 0:5:100;
CRLB = {zeros(4,numel(SNR)),zeros(4,numel(SNR)),...
        zeros(4,numel(SNR))};
for k=1:numel(SNR)
    signal1.remove_noise;
    signal2.remove_noise;
    signal3.remove_noise;
    sigma1 = signal1.add_white_gaussian_noise(SNR(k),'db');
    sigma2 = signal2.add_white_gaussian_noise(SNR(k),'db');
    sigma3 = signal3.add_white_gaussian_noise(SNR(k),'db');
    [a1,d1,f1,p1] = params1.crlb(sigma1,nsamples(1));
    [a2,d2,f2,p2] = params2.crlb(sigma2,nsamples(2));
    [a3,d3,f3,p3] = params3.crlb(sigma3,nsamples(3));
    M1 = [a1;d1;f1;p1];
    M2 = [a2;d2;f2;p2];
    M3 = [a3;d3;f3;p3];
    CRLB{1}(:,k) = sqrt(mean(M1.^2,2));
    CRLB{2}(:,k) = sqrt(mean(M2.^2,2));
    CRLB{3}(:,k) = sqrt(mean(M3.^2,2));
end

for k = [3,1,2,4]
    figure
    semilogy(SNR,CRLB{1}(k,:),'b-^')    
    hold on
    semilogy(SNR,CRLB{2}(k,:),'g-s')
    semilogy(SNR,CRLB{3}(k,:),'r-o')
end
figure(1)
title(['Figure 3. Root mean square of the CRLB vector of the '...
       '\omega_i, i=1,...,10, respectively for \Omega=100, '...
       'N=200 (blue) \Omega=10, N=200 (green), \Omega=10, '...
       'N=20 (red).'])
figure(2)
title(['Root mean square of the CRLB vector of the \beta_i, '...
       'i=1,...,10, respectively for \Omega=100, N=200 (blue)'...
       ' \Omega=10, N=200 (green), \Omega=10, N=20 (red).'])
figure(3)
title(['Root mean square of the CRLB vector of the \psi_i, '...
       'i=1,...,10, respectively for \Omega=100, N=200 (blue)'...
       ' \Omega=10, N=200 (green), \Omega=10, N=20 (red).'])
figure(4)
title(['Root mean square of the CRLB vector of the \gamma_i, '...
       'i=1,...,10, respectively for \Omega=100, N=200 (blue)'...
       ' \Omega=10, N=200 (green), \Omega=10, N=20 (red).'])